@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import pickle
+import socket
 from email.errors import MessageDefect
 from test import DUMMY_POOL
 
@@ -17,6 +18,7 @@ from urllib3.exceptions import (
     HTTPError,
     LocationParseError,
     MaxRetryError,
+    NameResolutionError,
     NewConnectionError,
     ReadTimeoutError,
 )
@@ -38,6 +40,8 @@ class TestPickle:
             EmptyPoolError(HTTPConnectionPool("localhost"), ""),
             HostChangedError(HTTPConnectionPool("localhost"), "/", 0),
             ReadTimeoutError(HTTPConnectionPool("localhost"), "/", ""),
+            NewConnectionError(HTTPConnection("localhost"), ""),
+            NameResolutionError("", HTTPConnection("localhost"), socket.gaierror()),
         ],
     )
     def test_exceptions(self, exception: Exception) -> None:
