@@ -4,19 +4,17 @@ import platform
 import select
 import socket
 import ssl
+import threading
 import typing
 from unittest import mock
 
 import pytest
 
-from dummyserver.server import DEFAULT_CA, DEFAULT_CERTS
+from dummyserver.socketserver import DEFAULT_CA, DEFAULT_CERTS
 from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
 from urllib3.util import ssl_
 from urllib3.util.ssltransport import SSLTransport
 
-if typing.TYPE_CHECKING:
-    from typing_extensions import Literal
-
 # consume_socket can iterate forever, we add timeouts to prevent halting.
 PER_TEST_TIMEOUT = 60
 
@@ -34,13 +32,11 @@ def server_client_ssl_contexts() -> tuple[ssl.SSLContext, ssl.SSLContext]:
 
 
 @typing.overload
-def sample_request(binary: Literal[True] = ...) -> bytes:
-    ...
+def sample_request(binary: typing.Literal[True] = ...) -> bytes: ...
 
 
 @typing.overload
-def sample_request(binary: Literal[False]) -> str:
-    ...
+def sample_request(binary: typing.Literal[False]) -> str: ...
 
 
 def sample_request(binary: bool = True) -> bytes | str:
@@ -54,7 +50,7 @@ def sample_request(binary: bool = True) -> bytes | str:
 
 
 def validate_request(
-    provided_request: bytearray, binary: Literal[False, True] = True
+    provided_request: bytearray, binary: typing.Literal[False, True] = True
 ) -> None:
     assert provided_request is not None
     expected_request = sample_request(binary)
@@ -62,18 +58,15 @@ def validate_request(
 
 
 @typing.overload
-def sample_response(binary: Literal[True] = ...) -> bytes:
-    ...
+def sample_response(binary: typing.Literal[True] = ...) -> bytes: ...
 
 
 @typing.overload
-def sample_response(binary: Literal[False]) -> str:
-    ...
+def sample_response(binary: typing.Literal[False]) -> str: ...
 
 
 @typing.overload
-def sample_response(binary: bool = ...) -> bytes | str:
-    ...
+def sample_response(binary: bool = ...) -> bytes | str: ...
 
 
 def sample_response(binary: bool = True) -> bytes | str:
@@ -91,11 +84,11 @@ def validate_response(
 
 def validate_peercert(ssl_socket: SSLTransport) -> None:
     binary_cert = ssl_socket.getpeercert(binary_form=True)
-    assert type(binary_cert) == bytes
+    assert type(binary_cert) is bytes
     assert len(binary_cert) > 0
 
     cert = ssl_socket.getpeercert()
-    assert type(cert) == dict
+    assert type(cert) is dict
     assert "serialNumber" in cert
     assert cert["serialNumber"] != ""
 
@@ -111,20 +104,29 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
         cls.server_context, cls.client_context = server_client_ssl_contexts()
 
     def start_dummy_server(
-        self, handler: typing.Callable[[socket.socket], None] | None = None
+        self,
+        handler: typing.Callable[[socket.socket], None] | None = None,
+        validate: bool = True,
     ) -> None:
+        quit_event = threading.Event()
+
         def socket_handler(listener: socket.socket) -> None:
             sock = listener.accept()[0]
             try:
                 with self.server_context.wrap_socket(sock, server_side=True) as ssock:
-                    request = consume_socket(ssock)
+                    request = consume_socket(
+                        ssock,
+                        quit_event=quit_event,
+                    )
+                    if not validate:
+                        return
                     validate_request(request)
                     ssock.send(sample_response())
             except (ConnectionAbortedError, ConnectionResetError):
                 return
 
         chosen_handler = handler if handler else socket_handler
-        self._start_server(chosen_handler)
+        self._start_server(chosen_handler, quit_event=quit_event)
 
     @pytest.mark.timeout(PER_TEST_TIMEOUT)
     def test_start_closed_socket(self) -> None:
@@ -138,7 +140,7 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
     @pytest.mark.timeout(PER_TEST_TIMEOUT)
     def test_close_after_handshake(self) -> None:
         """Socket errors should be bubbled up"""
-        self.start_dummy_server()
+        self.start_dummy_server(validate=False)
 
         sock = socket.create_connection((self.host, self.port))
         with SSLTransport(
@@ -184,33 +186,33 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
         """
 
         def shutdown_handler(listener: socket.socket) -> None:
-            sock = listener.accept()[0]
-            ssl_sock = self.server_context.wrap_socket(sock, server_side=True)
+            with (
+                listener.accept()[0] as sock,
+                self.server_context.wrap_socket(sock, server_side=True) as ssl_sock,
+            ):
+                request = consume_socket(ssl_sock)
+                validate_request(request)
+                ssl_sock.sendall(sample_response())
 
-            request = consume_socket(ssl_sock)
-            validate_request(request)
-            ssl_sock.sendall(sample_response())
-
-            unwrapped_sock = ssl_sock.unwrap()
-
-            request = consume_socket(unwrapped_sock)
-            validate_request(request)
-            unwrapped_sock.sendall(sample_response())
+                with ssl_sock.unwrap() as unwrapped_sock:
+                    request = consume_socket(unwrapped_sock)
+                    validate_request(request)
+                    unwrapped_sock.sendall(sample_response())
 
         self.start_dummy_server(shutdown_handler)
-        sock = socket.create_connection((self.host, self.port))
-        ssock = SSLTransport(sock, self.client_context, server_hostname="localhost")
+        with socket.create_connection((self.host, self.port)) as sock:
+            ssock = SSLTransport(sock, self.client_context, server_hostname="localhost")
 
-        # request/response over TLS.
-        ssock.sendall(sample_request())
-        response = consume_socket(ssock)
-        validate_response(response)
+            # request/response over TLS.
+            ssock.sendall(sample_request())
+            response = consume_socket(ssock)
+            validate_response(response)
 
-        # request/response over plaintext after unwrap.
-        ssock.unwrap()
-        sock.sendall(sample_request())
-        response = consume_socket(sock)
-        validate_response(response)
+            # request/response over plaintext after unwrap.
+            ssock.unwrap()
+            sock.sendall(sample_request())
+            response = consume_socket(sock)
+            validate_response(response)
 
     @pytest.mark.timeout(PER_TEST_TIMEOUT)
     def test_ssl_object_attributes(self) -> None:
@@ -222,11 +224,10 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
             sock, self.client_context, server_hostname="localhost"
         ) as ssock:
             cipher = ssock.cipher()
-            assert type(cipher) == tuple
+            assert type(cipher) is tuple
 
-            # No chosen protocol through ALPN or NPN.
+            # No chosen protocol.
             assert ssock.selected_alpn_protocol() is None
-            assert ssock.selected_npn_protocol() is None
 
             shared_ciphers = ssock.shared_ciphers()
             # SSLContext.shared_ciphers() changed behavior completely in a patch version.
@@ -443,7 +444,7 @@ class TlsInTlsTestCase(SocketDummyServerTestCase):
                 proxy_sock, self.client_context, server_hostname="localhost"
             ) as destination_sock:
                 file = destination_sock.makefile("rwb", buffering)
-                file.write(sample_request())  # type: ignore[call-overload]
+                file.write(sample_request())  # type: ignore[arg-type]
                 file.flush()
 
                 response = bytearray(65536)
@@ -484,11 +485,11 @@ class TlsInTlsTestCase(SocketDummyServerTestCase):
                 write.flush()
 
                 response = read.read()
-                assert isinstance(response, str)
+                assert type(response) is str
                 if "\r" not in response:
                     # Carriage return will be removed when reading as a file on
                     # some platforms.  We add it before the comparison.
-                    assert isinstance(response, str)
+                    assert type(response) is str
                     response = response.replace("\n", "\r\n")
                 validate_response(response, binary=False)
 
