@@ -1,21 +1,21 @@
 from __future__ import annotations
 
+import concurrent.futures
 import contextlib
 import datetime
 import os.path
 import shutil
 import ssl
-import sys
 import tempfile
+import time
+import typing
 import warnings
 from pathlib import Path
 from test import (
     LONG_TIMEOUT,
     SHORT_TIMEOUT,
     TARPIT_HOST,
-    notSecureTransport,
     requires_network,
-    requires_ssl_context_keyfile_password,
     resolvesLocalhostFQDN,
 )
 from test.conftest import ServerConfig
@@ -24,15 +24,17 @@ from unittest import mock
 import pytest
 import trustme
 
+import urllib3.http2
+import urllib3.http2.probe as http2_probe
 import urllib3.util as util
 import urllib3.util.ssl_
-from dummyserver.server import (
+from dummyserver.socketserver import (
     DEFAULT_CA,
     DEFAULT_CA_KEY,
     DEFAULT_CERTS,
     encrypt_key_pem,
 )
-from dummyserver.testcase import HTTPSDummyServerTestCase
+from dummyserver.testcase import HTTPSHypercornDummyServerTestCase
 from urllib3 import HTTPSConnectionPool
 from urllib3.connection import RECENT_DATE, HTTPSConnection, VerifiedHTTPSConnection
 from urllib3.exceptions import (
@@ -46,8 +48,6 @@ from urllib3.exceptions import (
 from urllib3.util.ssl_match_hostname import CertificateError
 from urllib3.util.timeout import Timeout
 
-from .. import has_alpn
-
 TLSv1_CERTS = DEFAULT_CERTS.copy()
 TLSv1_CERTS["ssl_version"] = getattr(ssl, "PROTOCOL_TLSv1", None)
 
@@ -68,7 +68,7 @@ PASSWORD_CLIENT_KEYFILE = "client_password.key"
 CLIENT_CERT = CLIENT_INTERMEDIATE_PEM
 
 
-class TestHTTPS(HTTPSDummyServerTestCase):
+class BaseTestHTTPS(HTTPSHypercornDummyServerTestCase):
     tls_protocol_name: str | None = None
 
     def tls_protocol_not_default(self) -> bool:
@@ -86,11 +86,17 @@ class TestHTTPS(HTTPSDummyServerTestCase):
     def ssl_version(self) -> int:
         if self.tls_protocol_name is None:
             return pytest.skip("Skipping base test class")
-        attribute = f"PROTOCOL_{self.tls_protocol_name.replace('.', '_')}"
-        ssl_version = getattr(ssl, attribute, None)
-        if ssl_version is None:
-            return pytest.skip(f"ssl.{attribute} isn't available")
-        return ssl_version  # type: ignore[no-any-return]
+
+        if self.tls_protocol_name == "TLSv1.3" and ssl.HAS_TLSv1_3:
+            return ssl.PROTOCOL_TLS_CLIENT
+        if self.tls_protocol_name == "TLSv1.2" and ssl.HAS_TLSv1_2:
+            return ssl.PROTOCOL_TLSv1_2
+        if self.tls_protocol_name == "TLSv1.1" and ssl.HAS_TLSv1_1:
+            return ssl.PROTOCOL_TLSv1_1
+        if self.tls_protocol_name == "TLSv1" and ssl.HAS_TLSv1:
+            return ssl.PROTOCOL_TLSv1
+        else:
+            return pytest.skip(f"{self.tls_protocol_name} isn't available")
 
     @classmethod
     def setup_class(cls) -> None:
@@ -132,7 +138,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
         shutil.rmtree(cls.certs_dir)
 
-    def test_simple(self) -> None:
+    def test_simple(self, http_version: str) -> None:
         with HTTPSConnectionPool(
             self.host,
             self.port,
@@ -141,6 +147,12 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             r = https_pool.request("GET", "/")
             assert r.status == 200, r.data
+            assert r.headers["server"] == f"hypercorn-{http_version}"
+            assert r.data == b"Dummy server!"
+
+    def test_default_port(self) -> None:
+        conn = HTTPSConnection(self.host, port=None)
+        assert conn.port == 443
 
     @resolvesLocalhostFQDN()
     def test_dotted_fqdn(self) -> None:
@@ -190,7 +202,6 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             with pytest.raises((SSLError, ProtocolError)):
                 https_pool.request("GET", "/certificate", retries=False)
 
-    @requires_ssl_context_keyfile_password()
     def test_client_key_password(self) -> None:
         with HTTPSConnectionPool(
             self.host,
@@ -205,7 +216,6 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             subject = r.json()
             assert subject["organizationalUnitName"].startswith("Testing cert")
 
-    @requires_ssl_context_keyfile_password()
     def test_client_encrypted_key_requires_password(self) -> None:
         with HTTPSConnectionPool(
             self.host,
@@ -218,7 +228,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             with pytest.raises(MaxRetryError, match="password is required") as e:
                 https_pool.request("GET", "/certificate")
 
-            assert isinstance(e.value.reason, SSLError)
+            assert type(e.value.reason) is SSLError
 
     def test_verified(self) -> None:
         with HTTPSConnectionPool(
@@ -228,8 +238,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=DEFAULT_CA,
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
@@ -243,8 +253,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         )
         ctx.load_verify_locations(cafile=DEFAULT_CA)
         with HTTPSConnectionPool(self.host, self.port, ssl_context=ctx) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with mock.patch("warnings.warn") as warn:
                 r = https_pool.request("GET", "/")
@@ -258,15 +268,14 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_context=ctx
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with mock.patch("warnings.warn") as warn:
                 r = https_pool.request("GET", "/")
                 assert r.status == 200
                 assert not warn.called, warn.call_args_list
 
-    @notSecureTransport()  # SecureTransport does not support cert directories
     def test_ca_dir_verified(self, tmp_path: Path) -> None:
         # OpenSSL looks up certificates by the hash for their name, see c_rehash
         # TODO infer the bytes using `cryptography.x509.Name.public_bytes`.
@@ -280,8 +289,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_cert_dir=str(tmp_path),
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
@@ -299,7 +308,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             with pytest.raises(MaxRetryError) as e:
                 https_pool.request("GET", "/", retries=0)
-            assert isinstance(e.value.reason, SSLError)
+            assert type(e.value.reason) is SSLError
             assert "doesn't match" in str(
                 e.value.reason
             ) or "certificate verify failed" in str(e.value.reason)
@@ -314,7 +323,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             with pytest.raises(MaxRetryError) as e:
                 https_pool.request("GET", "/")
-            assert isinstance(e.value.reason, SSLError)
+            assert type(e.value.reason) is SSLError
             assert (
                 "certificate verify failed" in str(e.value.reason)
                 # PyPy is more specific
@@ -329,14 +338,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=self.bad_ca_path,
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with pytest.raises(ssl.SSLError):
                     conn.connect()
 
                 assert conn.sock is not None  # type: ignore[attr-defined]
-            finally:
-                conn.close()
 
     def test_verified_without_ca_certs(self) -> None:
         # default is cert_reqs=None which is ssl.CERT_NONE
@@ -348,7 +354,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             with pytest.raises(MaxRetryError) as e:
                 https_pool.request("GET", "/")
-            assert isinstance(e.value.reason, SSLError)
+            assert type(e.value.reason) is SSLError
             # there is a different error message depending on whether or
             # not pyopenssl is injected
             assert (
@@ -453,6 +459,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             # the python ssl module).
             if hasattr(conn.sock, "server_hostname"):  # type: ignore[attr-defined]
                 assert conn.sock.server_hostname == "localhost"  # type: ignore[attr-defined]
+            conn.getresponse().close()
+            conn.close()
 
     def test_assert_fingerprint_md5(self) -> None:
         with HTTPSConnectionPool(
@@ -496,7 +504,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         def _test_request(pool: HTTPSConnectionPool) -> SSLError:
             with pytest.raises(MaxRetryError) as cm:
                 pool.request("GET", "/", retries=0)
-            assert isinstance(cm.value.reason, SSLError)
+            assert type(cm.value.reason) is SSLError
             return cm.value.reason
 
         with HTTPSConnectionPool(
@@ -539,7 +547,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             with pytest.raises(MaxRetryError) as cm:
                 https_pool.request("GET", "/", retries=0)
-            assert isinstance(cm.value.reason, SSLError)
+            assert type(cm.value.reason) is SSLError
 
     def test_verify_none_and_good_fingerprint(self) -> None:
         with HTTPSConnectionPool(
@@ -553,12 +561,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             https_pool.request("GET", "/")
 
-    @notSecureTransport()
     def test_good_fingerprint_and_hostname_mismatch(self) -> None:
-        # This test doesn't run with SecureTransport because we don't turn off
-        # hostname validation without turning off all validation, which this
-        # test doesn't do (deliberately). We should revisit this if we make
-        # new decisions.
         with HTTPSConnectionPool(
             "127.0.0.1",
             self.port,
@@ -613,7 +616,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             with pytest.warns(InsecureRequestWarning):
                 https_pool.request("GET", "/")
 
-    def test_tunnel(self) -> None:
+    def test_tunnel(self, http_version: str) -> None:
         """test the _tunnel behavior"""
         timeout = Timeout(total=None)
         with HTTPSConnectionPool(
@@ -623,17 +626,22 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             cert_reqs="CERT_NONE",
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
-                conn.set_tunnel(self.host, self.port)
-                with mock.patch.object(
-                    conn, "_tunnel", create=True, return_value=None
-                ) as conn_tunnel:
-                    with pytest.warns(InsecureRequestWarning):
-                        https_pool._make_request(conn, "GET", "/")
-                conn_tunnel.assert_called_once_with()
-            finally:
-                conn.close()
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                if http_version == "h2":
+                    with pytest.raises(NotImplementedError) as e:
+                        conn.set_tunnel(self.host, self.port)
+                    assert (
+                        str(e.value)
+                        == "HTTP/2 does not support setting up a tunnel through a proxy"
+                    )
+                else:
+                    conn.set_tunnel(self.host, self.port)
+                    with mock.patch.object(
+                        conn, "_tunnel", create=True, return_value=None
+                    ) as conn_tunnel:
+                        with pytest.warns(InsecureRequestWarning):
+                            https_pool._make_request(conn, "GET", "/")
+                    conn_tunnel.assert_called_once_with()
 
     @requires_network()
     def test_enhanced_timeout(self) -> None:
@@ -644,14 +652,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             retries=False,
             cert_reqs="CERT_REQUIRED",
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
+            with contextlib.closing(https_pool._new_conn()) as conn:
                 with pytest.raises(ConnectTimeoutError):
                     https_pool.request("GET", "/")
                 with pytest.raises(ConnectTimeoutError):
                     https_pool._make_request(conn, "GET", "/")
-            finally:
-                conn.close()
 
         with HTTPSConnectionPool(
             TARPIT_HOST,
@@ -670,14 +675,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             retries=False,
             cert_reqs="CERT_REQUIRED",
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
+            with contextlib.closing(https_pool._new_conn()) as conn:
                 with pytest.raises(ConnectTimeoutError):
                     https_pool.request(
                         "GET", "/", timeout=Timeout(total=None, connect=SHORT_TIMEOUT)
                     )
-            finally:
-                conn.close()
 
     def test_enhanced_ssl_connection(self) -> None:
         fingerprint = "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
@@ -753,9 +755,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             https_pool.ssl_version = ssl_version = self.certs["ssl_version"]
             if ssl_version is getattr(ssl, "PROTOCOL_TLS", object()):
-                cmgr: contextlib.AbstractContextManager[
-                    object
-                ] = contextlib.nullcontext()
+                cmgr: contextlib.AbstractContextManager[object] = (
+                    contextlib.nullcontext()
+                )
             else:
                 cmgr = pytest.warns(
                     DeprecationWarning,
@@ -805,29 +807,28 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             self.port,
             ca_certs=DEFAULT_CA,
             ssl_minimum_version=self.tls_version(),
+            ssl_maximum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 conn.connect()
                 if not hasattr(conn.sock, "version"):  # type: ignore[attr-defined]
                     pytest.skip("SSLSocket.version() not available")
                 assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
-            finally:
-                conn.close()
 
     def test_ssl_version_is_deprecated(self) -> None:
         if self.tls_protocol_name is None:
             pytest.skip("Skipping base test class")
+        if self.ssl_version() == ssl.PROTOCOL_TLS_CLIENT:
+            pytest.skip(
+                "Skipping because ssl_version=ssl.PROTOCOL_TLS_CLIENT is not deprecated"
+            )
 
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_version=self.ssl_version()
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with pytest.warns(DeprecationWarning) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert len(w) >= 1
         assert any(x.category == DeprecationWarning for x in w)
@@ -856,12 +857,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_version=ssl_version
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with warnings.catch_warnings(record=True) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert [str(wm) for wm in w if wm.category != ResourceWarning] == []
 
@@ -877,12 +875,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=DEFAULT_CA,
             ssl_context=ctx,
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with warnings.catch_warnings(record=True) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert [str(wm) for wm in w if wm.category != ResourceWarning] == []
 
@@ -909,11 +904,14 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 conn = https_pool._get_conn()
                 try:
                     conn.connect()
-                    assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
+                    if maximum_version == TLSVersion.MAXIMUM_SUPPORTED:
+                        # A higher protocol than tls_protocol_name could be negotiated
+                        assert conn.sock.version() >= self.tls_protocol_name  # type: ignore[attr-defined]
+                    else:
+                        assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
                 finally:
                     conn.close()
 
-    @pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python 3.8+")
     def test_sslkeylogfile(
         self, tmp_path: Path, monkeypatch: pytest.MonkeyPatch
     ) -> None:
@@ -959,10 +957,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             r = pool.request("GET", "/")
             assert r.status == 200, r.data
 
-    def test_alpn_default(self) -> None:
+    def test_alpn_default(self, http_version: str) -> None:
         """Default ALPN protocols are sent by default."""
-        if not has_alpn() or not has_alpn(ssl.SSLContext):
-            pytest.skip("ALPN-support not available")
         with HTTPSConnectionPool(
             self.host,
             self.port,
@@ -972,18 +968,154 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             r = pool.request("GET", "/alpn_protocol", retries=0)
             assert r.status == 200
             assert r.data.decode("utf-8") == util.ALPN_PROTOCOLS[0]
+            assert (
+                r.data.decode("utf-8") == {"h11": "http/1.1", "h2": "h2"}[http_version]
+            )
+
+    def test_http2_probe_result_is_cached(self, http_version: str) -> None:
+        assert http2_probe._values() == {}
+
+        for i in range(2):  # Do this twice to exercise the cache path
+            with HTTPSConnectionPool(
+                self.host,
+                self.port,
+                ca_certs=DEFAULT_CA,
+            ) as pool:
+                r = pool.request("GET", "/alpn_protocol", retries=0)
+                assert r.status == 200
+
+            if http_version == "h2":
+                # This means the probe was successful.
+                assert http2_probe._values() == {(self.host, self.port): True}
+            else:
+                # This means the probe wasn't attempted, otherwise would have a value.
+                assert http_version == "h11"
+                assert http2_probe._values() == {}
+
+    @pytest.mark.xfail(reason="Hypercorn always supports both HTTP/2 and HTTP/1.1")
+    def test_http2_probe_result_failed(self, http_version: str) -> None:
+        if http_version == "h2":
+            pytest.skip("Test must have server in HTTP/1.1 mode")
+        assert http2_probe._values() == {}
+
+        urllib3.http2.inject_into_urllib3()
+        try:
+            with HTTPSConnectionPool(
+                self.host,
+                self.port,
+                ca_certs=DEFAULT_CA,
+            ) as pool:
+                r = pool.request("GET", "/", retries=0)
+                assert r.status == 200
+
+            # The probe was a failure because Hypercorn didn't support HTTP/2.
+            assert http2_probe._values() == {(self.host, self.port): False}
+        finally:
+            urllib3.http2.extract_from_urllib3()
+
+    def test_http2_probe_no_result_in_connect_error(self) -> None:
+        assert http2_probe._values() == {}
+
+        urllib3.http2.inject_into_urllib3()
+        try:
+            with HTTPSConnectionPool(
+                TARPIT_HOST,
+                self.port,
+                ca_certs=DEFAULT_CA,
+                timeout=SHORT_TIMEOUT,
+            ) as pool:
+                with pytest.raises(ConnectTimeoutError):
+                    pool.request("GET", "/", retries=False)
+
+            # The probe was inconclusive since an error occurred during connection.
+            assert http2_probe._values() == {(TARPIT_HOST, self.port): None}
+        finally:
+            urllib3.http2.extract_from_urllib3()
+
+    def test_http2_probe_no_result_in_ssl_error(self) -> None:
+        urllib3.http2.inject_into_urllib3()
+        try:
+            with HTTPSConnectionPool(
+                self.host,
+                self.port,
+                ca_certs=None,
+                timeout=LONG_TIMEOUT,
+            ) as pool:
+                with pytest.raises(SSLError):
+                    pool.request("GET", "/", retries=False)
+
+            # The probe was inconclusive since an error occurred during connection.
+            assert http2_probe._values() == {(self.host, self.port): None}
+        finally:
+            urllib3.http2.extract_from_urllib3()
+
+    def test_http2_probe_blocked_per_thread(self) -> None:
+        state, current_thread, last_action = None, None, time.perf_counter()
+
+        def connect_callback(label: str, thread_id: int, **kwargs: typing.Any) -> None:
+            nonlocal state, current_thread, last_action
+
+            if label in ("before connect", "after connect failure"):
+                # We don't know if the target supports HTTP/2 as connections fail
+                assert kwargs["target_supports_http2"] is None
+
+            # Since we're trying to connect to TARPIT_HOST, all connections will
+            # fail, but they should be tried one after the other
+            now = time.perf_counter()
+            assert now >= last_action
+            last_action = now
+
+            if label == "before connect":
+                assert state is None
+                state = "connect"
+                assert current_thread != thread_id
+                current_thread = thread_id
+            elif label == "after connect failure":
+                assert state == "connect"
+                assert current_thread == thread_id
+                state = None
+
+        assert http2_probe._values() == {}
+
+        connect_timeout = LONG_TIMEOUT
+        total_threads = 3
+        urllib3.http2.inject_into_urllib3()
+        try:
+
+            def try_connect(_: typing.Any) -> tuple[float, float]:
+                with HTTPSConnectionPool(
+                    TARPIT_HOST,
+                    self.port,
+                    ca_certs=DEFAULT_CA,
+                    timeout=connect_timeout,
+                ) as pool:
+                    start_time = time.time()
+                    conn = pool._get_conn()
+                    assert isinstance(conn, HTTPSConnection)
+                    conn._connect_callback = connect_callback
+                    with pytest.raises(ConnectTimeoutError):
+                        conn.connect()
+                    end_time = time.time()
+                    return start_time, end_time
+
+            threadpool = concurrent.futures.ThreadPoolExecutor(total_threads)
+            list(threadpool.map(try_connect, range(total_threads)))
+
+            # The probe was inconclusive since an error occurred during connection.
+            assert http2_probe._values() == {(TARPIT_HOST, self.port): None}
+        finally:
+            urllib3.http2.extract_from_urllib3()
 
     def test_default_ssl_context_ssl_min_max_versions(self) -> None:
         ctx = urllib3.util.ssl_.create_urllib3_context()
         assert ctx.minimum_version == ssl.TLSVersion.TLSv1_2
         # urllib3 sets a default maximum version only when it is
-        # injected with PyOpenSSL- or SecureTransport-backed
-        # SSL-support.
+        # injected with PyOpenSSL SSL-support.
         # Otherwise, the default maximum version is set by Python's
         # `ssl.SSLContext`. The value respects OpenSSL configuration and
         # can be different from `ssl.TLSVersion.MAXIMUM_SUPPORTED`.
         # https://github.com/urllib3/urllib3/issues/2477#issuecomment-1151452150
-        if util.IS_PYOPENSSL or util.IS_SECURETRANSPORT:
+        if util.IS_PYOPENSSL:
             expected_maximum_version = ssl.TLSVersion.MAXIMUM_SUPPORTED
         else:
             expected_maximum_version = ssl.SSLContext(
@@ -992,6 +1124,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         assert ctx.maximum_version == expected_maximum_version
 
     def test_ssl_context_ssl_version_uses_ssl_min_max_versions(self) -> None:
+        if self.ssl_version() == ssl.PROTOCOL_TLS_CLIENT:
+            pytest.skip(
+                "Skipping because ssl_version=ssl.PROTOCOL_TLS_CLIENT is not deprecated"
+            )
+
         with pytest.warns(
             DeprecationWarning,
             match=r"'ssl_version' option is deprecated and will be removed in "
@@ -1003,27 +1140,47 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         assert ctx.minimum_version == self.tls_version()
         assert ctx.maximum_version == self.tls_version()
 
+    def test_assert_missing_hashfunc(self, monkeypatch: pytest.MonkeyPatch) -> None:
+        fingerprint = "55:39:BF:70:05:12:43:FA:1F:D1:BF:4E:E8:1B:07:1D"
+        with HTTPSConnectionPool(
+            "localhost",
+            self.port,
+            cert_reqs="CERT_REQUIRED",
+            ca_certs=DEFAULT_CA,
+            assert_fingerprint=(fingerprint),
+            ssl_minimum_version=self.tls_version(),
+        ) as https_pool:
+            digest_length = len(fingerprint.replace(":", "").lower())
+            monkeypatch.setitem(urllib3.util.ssl_.HASHFUNC_MAP, digest_length, None)
+            with pytest.raises(MaxRetryError) as cm:
+                https_pool.request("GET", "/", retries=0)
+            assert type(cm.value.reason) is SSLError
+            assert (
+                f"Hash function implementation unavailable for fingerprint length: {digest_length}"
+                == str(cm.value.reason)
+            )
+
 
 @pytest.mark.usefixtures("requires_tlsv1")
-class TestHTTPS_TLSv1(TestHTTPS):
+class TestHTTPS_TLSv1(BaseTestHTTPS):
     tls_protocol_name = "TLSv1"
     certs = TLSv1_CERTS
 
 
 @pytest.mark.usefixtures("requires_tlsv1_1")
-class TestHTTPS_TLSv1_1(TestHTTPS):
+class TestHTTPS_TLSv1_1(BaseTestHTTPS):
     tls_protocol_name = "TLSv1.1"
     certs = TLSv1_1_CERTS
 
 
 @pytest.mark.usefixtures("requires_tlsv1_2")
-class TestHTTPS_TLSv1_2(TestHTTPS):
+class TestHTTPS_TLSv1_2(BaseTestHTTPS):
     tls_protocol_name = "TLSv1.2"
     certs = TLSv1_2_CERTS
 
 
 @pytest.mark.usefixtures("requires_tlsv1_3")
-class TestHTTPS_TLSv1_3(TestHTTPS):
+class TestHTTPS_TLSv1_3(BaseTestHTTPS):
     tls_protocol_name = "TLSv1.3"
     certs = TLSv1_3_CERTS
 
@@ -1112,7 +1269,7 @@ class TestHTTPS_Hostname:
         # IP addresses should fail for commonName.
         else:
             assert err is not None
-            assert type(err.reason) == SSLError
+            assert type(err.reason) is SSLError
             assert isinstance(
                 err.reason.args[0], (ssl.SSLCertVerificationError, CertificateError)
             )
@@ -1160,7 +1317,7 @@ class TestHTTPS_IPV4SAN:
 class TestHTTPS_IPV6SAN:
     @pytest.mark.parametrize("host", ["::1", "[::1]"])
     def test_can_validate_ipv6_san(
-        self, ipv6_san_server: ServerConfig, host: str
+        self, ipv6_san_server: ServerConfig, host: str, http_version: str
     ) -> None:
         """Ensure that urllib3 can validate SANs with IPv6 addresses in them."""
         with HTTPSConnectionPool(
@@ -1171,3 +1328,4 @@ class TestHTTPS_IPV6SAN:
         ) as https_pool:
             r = https_pool.request("GET", "/")
             assert r.status == 200
+            assert r.headers["server"] == f"hypercorn-{http_version}"
