@@ -14,6 +14,7 @@ import socket
 import ssl
 import tempfile
 import threading
+import time
 import typing
 import zlib
 from collections import OrderedDict
@@ -32,7 +33,13 @@ from dummyserver.socketserver import (
     get_unreachable_address,
 )
 from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
-from urllib3 import HTTPConnectionPool, HTTPSConnectionPool, ProxyManager, util
+from urllib3 import (
+    BaseHTTPResponse,
+    HTTPConnectionPool,
+    HTTPSConnectionPool,
+    ProxyManager,
+    util,
+)
 from urllib3._collections import HTTPHeaderDict
 from urllib3.connection import HTTPConnection, _get_default_user_agent
 from urllib3.connectionpool import _url_from_pool
@@ -960,25 +967,28 @@ class TestSocketClosing(SocketDummyServerTestCase):
             listener: socket.socket,
         ) -> None:
             try:
-                with listener.accept()[0] as sock, original_ssl_wrap_socket(
-                    sock,
-                    server_side=True,
-                    keyfile=DEFAULT_CERTS["keyfile"],
-                    certfile=DEFAULT_CERTS["certfile"],
-                    ca_certs=DEFAULT_CA,
-                ) as ssl_sock:
+                with (
+                    listener.accept()[0] as sock,
+                    original_ssl_wrap_socket(
+                        sock,
+                        server_side=True,
+                        keyfile=DEFAULT_CERTS["keyfile"],
+                        certfile=DEFAULT_CERTS["certfile"],
+                        ca_certs=DEFAULT_CA,
+                    ) as ssl_sock,
+                ):
                     consume_socket(ssl_sock, quit_event=quit_event)
             except (ConnectionResetError, ConnectionAbortedError, OSError):
                 pass
 
         self._start_server(consume_ssl_socket, quit_event=quit_event)
-        with socket.create_connection(
-            (self.host, self.port)
-        ) as sock, contextlib.closing(
-            ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
-        ) as ssl_sock, ssl_sock.makefile(
-            "rb"
-        ) as f:
+        with (
+            socket.create_connection((self.host, self.port)) as sock,
+            contextlib.closing(
+                ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
+            ) as ssl_sock,
+            ssl_sock.makefile("rb") as f,
+        ):
             ssl_sock.close()
             f.close()
             with pytest.raises(OSError):
@@ -990,30 +1000,94 @@ class TestSocketClosing(SocketDummyServerTestCase):
 
         def consume_ssl_socket(listener: socket.socket) -> None:
             try:
-                with listener.accept()[0] as sock, original_ssl_wrap_socket(
-                    sock,
-                    server_side=True,
-                    keyfile=DEFAULT_CERTS["keyfile"],
-                    certfile=DEFAULT_CERTS["certfile"],
-                    ca_certs=DEFAULT_CA,
-                ) as ssl_sock:
+                with (
+                    listener.accept()[0] as sock,
+                    original_ssl_wrap_socket(
+                        sock,
+                        server_side=True,
+                        keyfile=DEFAULT_CERTS["keyfile"],
+                        certfile=DEFAULT_CERTS["certfile"],
+                        ca_certs=DEFAULT_CA,
+                    ) as ssl_sock,
+                ):
                     consume_socket(ssl_sock, quit_event=quit_event)
             except (ConnectionResetError, ConnectionAbortedError, OSError):
                 pass
 
         self._start_server(consume_ssl_socket, quit_event=quit_event)
-        with socket.create_connection(
-            (self.host, self.port)
-        ) as sock, contextlib.closing(
-            ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
-        ) as ssl_sock, ssl_sock.makefile(
-            "rb"
+        with (
+            socket.create_connection((self.host, self.port)) as sock,
+            contextlib.closing(
+                ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
+            ) as ssl_sock,
+            ssl_sock.makefile("rb"),
         ):
             ssl_sock.close()
             ssl_sock.close()
             ssl_sock.sendall(b"hello")
             assert ssl_sock.fileno() > 0
 
+    def test_socket_shutdown_stops_recv(self) -> None:
+        timed_out, starting_read = Event(), Event()
+
+        def socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+
+            ssl_sock = original_ssl_wrap_socket(
+                sock,
+                server_side=True,
+                keyfile=DEFAULT_CERTS["keyfile"],
+                certfile=DEFAULT_CERTS["certfile"],
+                ca_certs=DEFAULT_CA,
+            )
+
+            # Consume request
+            buf = b""
+            while not buf.endswith(b"\r\n\r\n"):
+                buf = ssl_sock.recv(65535)
+
+            # Send incomplete message (note Content-Length)
+            ssl_sock.send(
+                b"HTTP/1.1 200 OK\r\n"
+                b"Content-Type: text/plain\r\n"
+                b"Content-Length: 10\r\n"
+                b"\r\n"
+                b"Hi-"
+            )
+            timed_out.wait(5)
+            ssl_sock.close()
+
+        self._start_server(socket_handler)
+
+        class TestClient(threading.Thread):
+            def __init__(self, host: str, port: int) -> None:
+                super().__init__()
+                self.host, self.port = host, port
+                self.response: BaseHTTPResponse | None = None
+
+            def run(self) -> None:
+                with HTTPSConnectionPool(
+                    self.host, self.port, ca_certs=DEFAULT_CA
+                ) as pool:
+                    self.response = pool.urlopen(
+                        "GET", "/", preload_content=False, retries=0
+                    )
+                    with pytest.raises(ProtocolError, match="Connection broken"):
+                        starting_read.set()
+                        self.response.read()
+
+        test_client = TestClient(self.host, self.port)
+        test_client.start()
+        # First, wait to make sure the client is really stuck reading
+        starting_read.wait(5)
+        time.sleep(LONG_TIMEOUT)
+        # Calling shutdown here calls shutdown() on the underlying socket,
+        # so that the remaining read will fail instead of blocking
+        # indefinitely
+        assert test_client.response is not None
+        test_client.response.shutdown()
+        timed_out.set()
+
 
 class TestProxyManager(SocketDummyServerTestCase):
     def test_simple(self) -> None:
@@ -2464,7 +2538,7 @@ class TestContentFraming(SocketDummyServerTestCase):
         body: typing.Any
         if body_type == "generator":
 
-            def body_generator() -> typing.Generator[bytes, None, None]:
+            def body_generator() -> typing.Generator[bytes]:
                 yield b"x" * 10
 
             body = body_generator()
@@ -2502,7 +2576,7 @@ class TestContentFraming(SocketDummyServerTestCase):
 
         if body_type == "generator":
 
-            def body_generator() -> typing.Generator[bytes, None, None]:
+            def body_generator() -> typing.Generator[bytes]:
                 yield b"x" * 10
 
             body = body_generator()
